<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Class for FPS-independent timing</H3>
            <p>Author: David Clark (da_clark at shaw.ca)<br>
            Submission date: April 09, 2001</p>
	  
            <p>Description: Provides fps-independent scaling for movement and duration timing</p>

            <p>Download: <a href="pytimer.tar.gz">pytimer.tar.gz</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: 2.0</p>

            <p>Comments: Here's another system for managing timing issues in a realtime game.  This code scales all movement, duration and interval calculations to compensate for the current framerate.  The disadvantage of this system is that movement can become choppy when framerates drop - a sprite moving at 200 pixels per second will move 60 pixels per frame if the framerate drops to 5 fps.  On the other hand, a constant speed will be maintained; the sprite will move at 200 pixels per second regardless of the framerate.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#! /usr/bin/env python</span>

<span class="pycomment">"""
ScaleClock - Allow your realtime application to self-adjust for fps changes.

There are two general approaches to handling timing issues in a game. The first
is to specify a maximum frame rate, and use pygame.time.delay() to make the length
of each frame as close to the preceeding one as possible. This has the advantage of
simplicity, but it means that a powerful machine will run at the same rate as a
slow one, reducing efficiency - much processor time could be wasted in delay loops.

This example demonstrates the alternative - running at the maximum possible framerate,
then scaling all events to match. This method is more complicated; each moving object
must have its movement rate scaled to compensate for longer or shorter frame durations.
In addition, each recurring event must take into account the length of each frame in
its timer.

Create a timer object, then call timer.begin before your main loop, and timer.tick()
during that loop.  You'll need to handle movement and interval issues as mentioned
above; check the example code at the bottom to see how to do it.
"""</span>

import pygame, sys

<span class="pydef">class FpsClock:</span>
    <span class="pydef">def __init__(self):</span>
        self.frame_duration = 0.000
        self.this_frame_time = 0
        self.last_frame_time = 0
        return
    
    <span class="pydef">def tick(self):</span>
        <span class="pycomment">"Call this every frame"</span>
        self.this_frame_time = self.get_current_time()
        self.frame_duration = (self.this_frame_time - self.last_frame_time) / 1000.000
        self.last_frame_time = self.this_frame_time
        return

    <span class="pydef">def get_frame_duration(self):</span>
        <span class="pycomment">"Returns the length of the previous frame, in seconds"</span>
        return self.frame_duration

    <span class="pydef">def get_current_time(self):</span>
        <span class="pycomment">"Used internally. Returns current time in ms."</span>
        return pygame.time.get_ticks()

    <span class="pydef">def begin(self):</span>
        <span class="pycomment">"Starts/restarts the timer. Call just before your main loop."</span>
        self.last_frame_time = self.get_current_time()
        return
        
if __name__ == "__main__":

    pygame.init()

    timer = FpsClock()

    timer.begin()

    <span class="pycomment"># Object setup for the movement example</span>
    
    display = pygame.display.set_mode((320, 20), pygame.SWSURFACE)

    red_square = pygame.Surface((20, 20), pygame.SWSURFACE)
    red_square.fill((255, 0, 0))
    
    black_square = pygame.Surface((20, 20), pygame.SWSURFACE)
    black_square.fill((0, 0, 0))

    square_speed = 160 
    square_x = 0

    <span class="pycomment"># Setup for the interval example</span>
    
    tick_interval = 1.000
    tick_time = 0

    <span class="pycomment"># Change this to see how different framerates result in consistent</span>
    <span class="pycomment"># motion. Try values like 50 or 200</span>
    
    delay_duration = 5
    
    while 1:
        <span class="pycomment"># Interval example</span>
        
        tick_time += timer.get_frame_duration()
        if tick_time > tick_interval:
            tick_time = 0
            print "<span class="pystring">Tick</span>"

        <span class="pycomment"># Motion example</span>
            
        display.blit(black_square, (square_x, 0))
        square_x += (square_speed * timer.get_frame_duration())
        if square_x > 320:
            square_x = 0
        display.blit(red_square, (square_x, 0))
        
        pygame.display.update()

        <span class="pycomment"># Insert artificial delay</span>
        
        pygame.time.delay(delay_duration)

        <span class="pycomment"># Must call this every frame</span>
        
        timer.tick()

        <span class="pycomment"># Check for keypress, exit if we get one.</span>
        
        pygame.event.peek()
        keys = pygame.key.get_pressed()
        for key in keys:
            if key:
                sys.exit()

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

